@@ -60,26 +60,45 @@
 #include <QToolButton>
 #include <QToolBar>
 #include <QPointer>
+#include <QMap>
+#include <QElapsedTimer>
+#include <QQuickWidget>
 
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+#include <QRandomGenerator>
+#endif
+
 #ifdef Q_OS_WIN
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QAxWidget>
 #endif
 #endif
 
-#include <QMap>
-#include <QElapsedTimer>
-
-#include "DockManager.h"
-#include "DockWidget.h"
-#include "DockAreaWidget.h"
-#include "DockAreaTitleBar.h"
 #include "DockAreaTabBar.h"
-#include "FloatingDockContainer.h"
+#include "DockAreaTitleBar.h"
+#include "DockAreaWidget.h"
 #include "DockComponentsFactory.h"
-#include "StatusDialog.h"
+#include "DockManager.h"
 #include "DockSplitter.h"
+#include "DockWidget.h"
+#include "FloatingDockContainer.h"
+#include "ImageViewer.h"
+#include "MyDockAreaTitleBar.h"
+#include "StatusDialog.h"
+
+/**
+ * Returns a random number from 0 to highest - 1
+ */
+int randomNumberBounded(int highest)
+{
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+	return QRandomGenerator::global()->bounded(highest);
+#else
+	qsrand(QTime::currentTime().msec());
+	return qrand() % highest;
+#endif
+}
 
 
 /**
@@ -127,7 +146,7 @@ public:
 	using Super = ads::CDockComponentsFactory;
 	ads::CDockAreaTitleBar* createDockAreaTitleBar(ads::CDockAreaWidget* DockArea) const override
 	{
-		auto TitleBar = new ads::CDockAreaTitleBar(DockArea);
+		auto TitleBar = new MyDockAreaTitleBar(DockArea);
 		auto CustomButton = new QToolButton(DockArea);
 		CustomButton->setToolTip(QObject::tr("Help"));
 		CustomButton->setIcon(svgIcon(":/adsdemo/images/help_outline.svg"));
@@ -216,9 +235,11 @@ struct MainWindowPrivate
 		QFileSystemModel* m = new QFileSystemModel(w);
 		m->setRootPath(QDir::currentPath());
 		w->setModel(m);
-		ads::CDockWidget* DockWidget = new ads::CDockWidget(QString("Filesystem %1")
+		w->setRootIndex(m->index(QDir::currentPath()));
+		ads::CDockWidget* DockWidget = DockManager->createDockWidget(QString("Filesystem %1")
 			.arg(FileSystemCount++));
 		DockWidget->setWidget(w);
+		DockWidget->setIcon(svgIcon(":/adsdemo/images/folder_open.svg"));
 		ui.menuView->addAction(DockWidget->toggleViewAction());
 		// We disable focus to test focus highlighting if the dock widget content
 		// does not support focus
@@ -236,7 +257,7 @@ struct MainWindowPrivate
 	{
 		static int CalendarCount = 0;
 		QCalendarWidget* w = new QCalendarWidget();
-		ads::CDockWidget* DockWidget = new ads::CDockWidget(QString("Calendar %1").arg(CalendarCount++));
+		ads::CDockWidget* DockWidget = DockManager->createDockWidget(QString("Calendar %1").arg(CalendarCount++));
 		// The following lines are for testing the setWidget() and takeWidget()
 		// functionality
 		DockWidget->setWidget(w);
@@ -249,6 +270,10 @@ struct MainWindowPrivate
 		auto ToolBar = DockWidget->createDefaultToolBar();
 		ToolBar->addAction(ui.actionSaveState);
 		ToolBar->addAction(ui.actionRestoreState);
+		// For testing all calendar dock widgets have a the tool button style
+		// Qt::ToolButtonTextUnderIcon
+		DockWidget->setToolBarStyleSource(ads::CDockWidget::ToolBarStyleFromDockWidget);
+		DockWidget->setToolBarStyle(Qt::ToolButtonTextUnderIcon, ads::CDockWidget::StateFloating);
 		return DockWidget;
 	}
 
@@ -277,8 +302,9 @@ struct MainWindowPrivate
 			.arg(LabelCount)
 			.arg(QTime::currentTime().toString("hh:mm:ss:zzz")));
 
-		ads::CDockWidget* DockWidget = new ads::CDockWidget(QString("Label %1").arg(LabelCount++));
+		ads::CDockWidget* DockWidget = DockManager->createDockWidget(QString("Label %1").arg(LabelCount++));
 		DockWidget->setWidget(l);
+		DockWidget->setIcon(svgIcon(":/adsdemo/images/font_download.svg"));
 		ui.menuView->addAction(DockWidget->toggleViewAction());
 		return DockWidget;
 	}
@@ -294,7 +320,7 @@ struct MainWindowPrivate
 		w->setPlaceholderText("This is an editor. If you close the editor, it will be "
 			"deleted. Enter your text here.");
 		w->setStyleSheet("border: none");
-		ads::CDockWidget* DockWidget = new ads::CDockWidget(QString("Editor %1").arg(EditorCount++));
+		ads::CDockWidget* DockWidget = DockManager->createDockWidget(QString("Editor %1").arg(EditorCount++));
 		DockWidget->setWidget(w);
 		DockWidget->setIcon(svgIcon(":/adsdemo/images/edit.svg"));
 		DockWidget->setFeature(ads::CDockWidget::CustomCloseHandling, true);
@@ -316,6 +342,35 @@ struct MainWindowPrivate
 		return DockWidget;
 	}
 
+	/**
+	 * Creates a simply image viewr
+	 */
+	ads::CDockWidget* createImageViewer()
+	{
+		static int ImageViewerCount = 0;
+		auto w = new CImageViewer();
+		auto ImageIndex = randomNumberBounded(4);
+		auto FileName = ":adsdemo/images/ads_logo.svg";
+
+		// Pick a random image from a number of images
+		switch (ImageIndex)
+		{
+		case 0: FileName = ":adsdemo/images/ads_tile_blue.svg"; break;
+		case 1: FileName = ":adsdemo/images/ads_tile_blue_light.svg"; break;
+		case 2: FileName = ":adsdemo/images/ads_tile_green.svg"; break;
+		case 3: FileName = ":adsdemo/images/ads_tile_orange.svg"; break;
+		}
+
+		auto Result = w->loadFile(FileName);
+		qDebug() << "loadFile result: " << Result;
+		ads::CDockWidget* DockWidget = DockManager->createDockWidget(QString("Image Viewer %1").arg(ImageViewerCount++));
+		DockWidget->setIcon(svgIcon(":/adsdemo/images/photo.svg"));
+		DockWidget->setWidget(w,ads:: CDockWidget::ForceNoScrollArea);
+		auto ToolBar = DockWidget->createDefaultToolBar();
+		ToolBar->addActions(w->actions());
+		return DockWidget;
+	}
+
 	/**
 	 * Create a table widget
 	 */
@@ -323,7 +378,7 @@ struct MainWindowPrivate
 	{
 		static int TableCount = 0;
 		auto w = new CMinSizeTableWidget();
-		ads::CDockWidget* DockWidget = new ads::CDockWidget(QString("Table %1").arg(TableCount++));
+		ads::CDockWidget* DockWidget = DockManager->createDockWidget(QString("Table %1").arg(TableCount++));
 		static int colCount = 5;
 		static int rowCount = 30;
 		w->setColumnCount(colCount);
@@ -341,7 +396,7 @@ struct MainWindowPrivate
 		DockWidget->setMinimumSizeHintMode(ads::CDockWidget::MinimumSizeHintFromContent);
 		auto ToolBar = DockWidget->createDefaultToolBar();
 		auto Action = ToolBar->addAction(svgIcon(":/adsdemo/images/fullscreen.svg"), "Toggle Fullscreen");
-		QObject::connect(Action, &QAction::triggered, [=]()
+		QObject::connect(Action, &QAction::triggered, [DockWidget]()
 			{
 				if (DockWidget->isFullScreen())
 				{
@@ -356,6 +411,17 @@ struct MainWindowPrivate
 		return DockWidget;
 	}
 
+	/**
+	 * Create QQuickWidget for test for OpenGL and QQuick
+	 */
+	ads::CDockWidget *createQQuickWidget()
+	{
+		QQuickWidget *widget = new QQuickWidget();
+		ads::CDockWidget *dockWidget = DockManager->createDockWidget("Quick");
+		dockWidget->setWidget(widget);
+		return dockWidget;
+	}
+
 
 #ifdef Q_OS_WIN
 #if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
@@ -366,14 +432,13 @@ struct MainWindowPrivate
 	{
 	   static int ActiveXCount = 0;
 	   QAxWidget* w = new QAxWidget("{6bf52a52-394a-11d3-b153-00c04f79faa6}", parent);
-	   ads::CDockWidget* DockWidget = new ads::CDockWidget(QString("Active X %1").arg(ActiveXCount++));
+	   ads::CDockWidget* DockWidget = DockManager->createDockWidget(QString("Active X %1").arg(ActiveXCount++));
 	   DockWidget->setWidget(w);
 	   ui.menuView->addAction(DockWidget->toggleViewAction());
 	   return DockWidget;
 	}
 #endif
 #endif
-
 };
 
 //============================================================================
@@ -386,8 +451,8 @@ void MainWindowPrivate::createContent()
 
 	// For this Special Dock Area we want to avoid dropping on the center of it (i.e. we don't want this widget to be ever tabbified):
 	{
-		SpecialDockArea->setAllowedAreas(ads::OuterDockAreas);
-		//SpecialDockArea->setAllowedAreas({ads::LeftDockWidgetArea, ads::RightDockWidgetArea}); // just for testing
+		//SpecialDockArea->setAllowedAreas(ads::OuterDockAreas);
+		SpecialDockArea->setAllowedAreas({ads::LeftDockWidgetArea, ads::RightDockWidgetArea, ads::TopDockWidgetArea}); // just for testing
 	}
 
 	DockWidget = createLongTextLabelDockWidget();
@@ -405,12 +470,12 @@ void MainWindowPrivate::createContent()
 	appendFeaturStringToWindowTitle(FileSystemWidget);
 
 	// Test custom factory - we inject a help button into the title bar
-	ads::CDockComponentsFactory::setFactory(new CCustomComponentsFactory());
+	DockManager->setComponentsFactory(new CCustomComponentsFactory());
 	auto TopDockArea = DockManager->addDockWidget(ads::TopDockWidgetArea, FileSystemWidget);
 	// Uncomment the next line if you would like to test the
 	// HideSingleWidgetTitleBar functionality
 	// TopDockArea->setDockAreaFlag(ads::CDockAreaWidget::HideSingleWidgetTitleBar, true);
-	ads::CDockComponentsFactory::resetDefaultFactory();
+	DockManager->setComponentsFactory(ads::CDockComponentsFactory::factory());
 
 	// We create a calendar widget and clear all flags to prevent the dock area
 	// from closing
@@ -445,7 +510,7 @@ void MainWindowPrivate::createContent()
 	auto TitleBar = DockArea->titleBar();
 	int Index = TitleBar->indexOf(TitleBar->tabBar());
 	TitleBar->insertWidget(Index + 1, CustomButton);
-	QObject::connect(CustomButton, &QToolButton::clicked, [=]()
+	QObject::connect(CustomButton, &QToolButton::clicked, [DockArea, this]()
 	{
 		auto DockWidget = createEditorWidget();
 		DockWidget->setFeature(ads::CDockWidget::DockWidgetDeleteOnClose, true);
@@ -455,15 +520,17 @@ void MainWindowPrivate::createContent()
 
 	// Test dock area docking
 	auto RighDockArea = DockManager->addDockWidget(ads::RightDockWidgetArea, createLongTextLabelDockWidget(), TopDockArea);
-	DockManager->addDockWidget(ads::TopDockWidgetArea, createLongTextLabelDockWidget(), RighDockArea);
+	DockWidget = createLongTextLabelDockWidget();
+	DockWidget->setFeature(ads::CDockWidget::DockWidgetPinnable, false);
+	DockManager->addDockWidget(ads::TopDockWidgetArea, DockWidget, RighDockArea);
 	auto BottomDockArea = DockManager->addDockWidget(ads::BottomDockWidgetArea, createLongTextLabelDockWidget(), RighDockArea);
 	DockManager->addDockWidget(ads::CenterDockWidgetArea, createLongTextLabelDockWidget(), RighDockArea);
 	auto LabelDockWidget = createLongTextLabelDockWidget();
-	std::cout << "DockWidget " << LabelDockWidget->objectName().toStdString() << std::endl;
 	DockManager->addDockWidget(ads::CenterDockWidgetArea, LabelDockWidget, BottomDockArea);
 
 	// Tests CustomCloseHandling without DeleteOnClose
 	LabelDockWidget->setFeature(ads::CDockWidget::CustomCloseHandling, true);
+	LabelDockWidget->setWindowTitle(LabelDockWidget->windowTitle() + " [Custom Close]");
 	QObject::connect(LabelDockWidget, &ads::CDockWidget::closeRequested, [LabelDockWidget, this]()
 	{
 		int Result = QMessageBox::question(_this, "Custom Close Request",
@@ -494,10 +561,7 @@ void MainWindowPrivate::createContent()
 
 #ifdef Q_OS_WIN
 #if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
-    if (!ads::CDockManager::testConfigFlag(ads::CDockManager::OpaqueUndocking))
-    {
-    	DockManager->addDockWidget(ads::CenterDockWidgetArea, createActiveXWidget(), RighDockArea);
-    }
+    DockManager->addDockWidget(ads::CenterDockWidgetArea, createActiveXWidget(), RighDockArea);
 #endif
 #endif
 
@@ -506,6 +570,15 @@ void MainWindowPrivate::createContent()
 		_this->connect(DockWidget, SIGNAL(viewToggled(bool)), SLOT(onViewToggled(bool)));
 		_this->connect(DockWidget, SIGNAL(visibilityChanged(bool)), SLOT(onViewVisibilityChanged(bool)));
 	}
+
+	// Create image viewer
+	DockWidget = createImageViewer();
+	DockManager->addDockWidget(ads::LeftDockWidgetArea, DockWidget);
+
+    // Create quick widget
+	DockWidget = createQQuickWidget();
+	DockWidget->setFeature(ads::CDockWidget::DockWidgetClosable, true);
+	DockManager->addDockWidget(ads::LeftDockWidgetArea, DockWidget);
 }
 
 
@@ -518,19 +591,29 @@ void MainWindowPrivate::createActions()
 	ui.toolBar->addAction(ui.actionRestoreState);
 	ui.actionRestoreState->setIcon(svgIcon(":/adsdemo/images/restore.svg"));
 
-	SavePerspectiveAction = new QAction("Create Perspective", _this);
-	SavePerspectiveAction->setIcon(svgIcon(":/adsdemo/images/picture_in_picture.svg"));
-	_this->connect(SavePerspectiveAction, SIGNAL(triggered()), SLOT(savePerspective()));
+	ui.toolBar->addSeparator();
+
+	QAction* a = ui.toolBar->addAction("Lock Workspace");
+	a->setIcon(svgIcon(":/adsdemo/images/lock_outline.svg"));
+	a->setCheckable(true);
+	a->setChecked(false);
+	QObject::connect(a, &QAction::triggered, _this, &CMainWindow::lockWorkspace);
+
 	PerspectiveListAction = new QWidgetAction(_this);
 	PerspectiveComboBox = new QComboBox(_this);
 	PerspectiveComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);
-	PerspectiveComboBox->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+	PerspectiveComboBox->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
 	PerspectiveListAction->setDefaultWidget(PerspectiveComboBox);
-	ui.toolBar->addSeparator();
 	ui.toolBar->addAction(PerspectiveListAction);
+
+	a = SavePerspectiveAction = ui.toolBar->addAction("Create Perspective");
+	a->setIcon(svgIcon(":/adsdemo/images/picture_in_picture.svg"));
+	QObject::connect(a, &QAction::triggered, _this, &CMainWindow::savePerspective);
 	ui.toolBar->addAction(SavePerspectiveAction);
 
-	QAction* a = ui.toolBar->addAction("Create Floating Editor");
+	ui.toolBar->addSeparator();
+
+	a = ui.toolBar->addAction("Create Floating Editor");
 	a->setProperty("Floating", true);
 	a->setToolTip("Creates floating dynamic dockable editor windows that are deleted on close");
 	a->setIcon(svgIcon(":/adsdemo/images/note_add.svg"));
@@ -544,12 +627,37 @@ void MainWindowPrivate::createActions()
 	_this->connect(a, SIGNAL(triggered()), SLOT(createEditor()));
 	ui.menuTests->addAction(a);
 
+	a = ui.toolBar->addAction("Create Editor Tab");
+	a->setProperty("Floating", false);
+	a->setToolTip("Creates a editor tab and inserts it as second tab into an area");
+	a->setIcon(svgIcon(":/adsdemo/images/tab.svg"));
+	a->setProperty("Tabbed", true);
+	_this->connect(a, SIGNAL(triggered()), SLOT(createEditor()));
+	ui.menuTests->addAction(a);
+
+	ui.toolBar->addSeparator();
 	a = ui.toolBar->addAction("Create Floating Table");
 	a->setToolTip("Creates floating dynamic dockable table with millions of entries");
 	a->setIcon(svgIcon(":/adsdemo/images/grid_on.svg"));
 	_this->connect(a, SIGNAL(triggered()), SLOT(createTable()));
 	ui.menuTests->addAction(a);
 
+	a = ui.toolBar->addAction("Create Image Viewer");
+	auto ToolButton = qobject_cast<QToolButton*>(ui.toolBar->widgetForAction(a));
+	ToolButton->setPopupMode(QToolButton::InstantPopup);
+	a->setToolTip("Creates floating, docked or pinned image viewer");
+	a->setIcon(svgIcon(":/adsdemo/images/panorama.svg"));
+	ui.menuTests->addAction(a);
+	auto Menu = new QMenu();
+	ToolButton->setMenu(Menu);
+	a = Menu->addAction("Floating Image Viewer");
+	_this->connect(a, SIGNAL(triggered()), SLOT(createImageViewer()));
+	a = Menu->addAction("Docked Image Viewer");
+	_this->connect(a, SIGNAL(triggered()), SLOT(createImageViewer()));
+	a = Menu->addAction("Pinned Image Viewer");
+	_this->connect(a, SIGNAL(triggered()), SLOT(createImageViewer()));
+
+
 	ui.menuTests->addSeparator();
 	a = ui.menuTests->addAction("Show Status Dialog");
 	_this->connect(a, SIGNAL(triggered()), SLOT(showStatusDialog()));
@@ -557,6 +665,12 @@ void MainWindowPrivate::createActions()
 	a = ui.menuTests->addAction("Toggle Label 0 Window Title");
 	_this->connect(a, SIGNAL(triggered()), SLOT(toggleDockWidgetWindowTitle()));
 	ui.menuTests->addSeparator();
+
+	a = ui.toolBar->addAction("Apply VS Style");
+	a->setToolTip("Applies a Visual Studio light style (visual_studio_light.css)." );
+	a->setIcon(svgIcon(":/adsdemo/images/color_lens.svg"));
+	QObject::connect(a, &QAction::triggered, _this, &CMainWindow::applyVsStyle);
+	ui.menuTests->addAction(a);
 }
 
 
@@ -616,7 +730,7 @@ CMainWindow::CMainWindow(QWidget *parent) :
 
     // uncomment the following line if you want to use opaque undocking and
 	// opaque splitter resizing
-    // CDockManager::setConfigFlags(CDockManager::DefaultOpaqueConfig);
+    //CDockManager::setConfigFlags(CDockManager::DefaultOpaqueConfig);
 
     // uncomment the following line if you want a fixed tab width that does
 	// not change if the visibility of the close button changes
@@ -646,11 +760,17 @@ CMainWindow::CMainWindow(QWidget *parent) :
 	// uncomment the following line if you want a central widget in the main dock container (the dock manager) without a titlebar
 	// If you enable this code, you can test it in the demo with the Calendar 0
 	// dock widget.
-	// CDockManager::setConfigFlag(CDockManager::HideSingleCentralWidgetTitleBar, true);
+	//CDockManager::setConfigFlag(CDockManager::HideSingleCentralWidgetTitleBar, true);
 
 	// uncomment the following line to enable focus highlighting of the dock
 	// widget that has the focus
-	CDockManager::setConfigFlag(CDockManager::FocusHighlighting, true);
+    CDockManager::setConfigFlag(CDockManager::FocusHighlighting, true);
+
+	// uncomment if you would like to enable dock widget auto hiding
+    CDockManager::setAutoHideConfigFlags({CDockManager::DefaultAutoHideConfig});
+
+    // uncomment if you would like to disable closing auto hide widget with mouse click outside of auto hide container
+    //CDockManager::setAutoHideConfigFlag(CDockManager::AutoHideCloseOnOutsideMouseClick, false);
 
 	// uncomment if you would like to enable an equal distribution of the
 	// available size of a splitter to all contained dock widgets
@@ -661,13 +781,14 @@ CMainWindow::CMainWindow(QWidget *parent) :
 
 	// Now create the dock manager and its content
 	d->DockManager = new CDockManager(this);
+	d->DockManager->setDockWidgetToolBarStyle(Qt::ToolButtonIconOnly, ads::CDockWidget::StateFloating);
 
  #if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
-	connect(d->PerspectiveComboBox, SIGNAL(activated(const QString&)),
-		d->DockManager, SLOT(openPerspective(const QString&)));
+	connect(d->PerspectiveComboBox, SIGNAL(activated(QString)),
+		d->DockManager, SLOT(openPerspective(QString)));
  #else
-    connect(d->PerspectiveComboBox, SIGNAL(textActivated(const QString&)),
-        d->DockManager, SLOT(openPerspective(const QString&)));
+    connect(d->PerspectiveComboBox, SIGNAL(textActivated(QString)),
+        d->DockManager, SLOT(openPerspective(QString)));
  #endif
 
 	d->createContent();
@@ -739,13 +860,14 @@ void CMainWindow::savePerspective()
 //============================================================================
 void CMainWindow::onViewToggled(bool Open)
 {
+	Q_UNUSED(Open);
 	auto DockWidget = qobject_cast<ads::CDockWidget*>(sender());
 	if (!DockWidget)
 	{
 		return;
 	}
 
-	qDebug() << DockWidget->objectName() << " viewToggled(" << Open << ")";
+	//qDebug() << DockWidget->objectName() << " viewToggled(" << Open << ")";
 }
 
 
@@ -769,6 +891,8 @@ void CMainWindow::createEditor()
 	QObject* Sender = sender();
 	QVariant vFloating = Sender->property("Floating");
 	bool Floating = vFloating.isValid() ? vFloating.toBool() : true;
+	QVariant vTabbed = Sender->property("Tabbed");
+	bool Tabbed = vTabbed.isValid() ? vTabbed.toBool() : true;
 	auto DockWidget = d->createEditorWidget();
 	DockWidget->setFeature(ads::CDockWidget::DockWidgetDeleteOnClose, true);
 	DockWidget->setFeature(ads::CDockWidget::DockWidgetForceCloseWithArea, true);
@@ -780,31 +904,38 @@ void CMainWindow::createEditor()
 		FloatingWidget->move(QPoint(20, 20));
 		d->LastCreatedFloatingEditor = DockWidget;
 		d->LastDockedEditor.clear();
+		return;
     }
-    else
-    {
-    	ads::CDockAreaWidget* EditorArea = d->LastDockedEditor ? d->LastDockedEditor->dockAreaWidget() : nullptr;
-    	if (EditorArea)
-    	{
-    		std::cout << "DockAreaCount before: " << EditorArea->dockContainer()->dockAreaCount() << std::endl;
-    		d->DockManager->setConfigFlag(ads::CDockManager::EqualSplitOnInsertion, true);
-    		d->DockManager->addDockWidget(ads::RightDockWidgetArea, DockWidget, EditorArea);
-    		std::cout << "DockAreaCount after: " << DockWidget->dockContainer()->dockAreaCount() << std::endl;
-    	}
-    	else
-    	{
-    		if (d->LastCreatedFloatingEditor)
-    		{
-    			std::cout << "LastCreated" << std::endl;
-    			d->DockManager->addDockWidget(ads::RightDockWidgetArea, DockWidget, d->LastCreatedFloatingEditor->dockAreaWidget());
-    		}
-    		else
-    		{
-    			d->DockManager->addDockWidget(ads::TopDockWidgetArea, DockWidget);
-    		}
-    	}
-    	d->LastDockedEditor = DockWidget;
-    }
+
+
+	ads::CDockAreaWidget* EditorArea = d->LastDockedEditor ? d->LastDockedEditor->dockAreaWidget() : nullptr;
+	if (EditorArea)
+	{
+		if (Tabbed)
+		{
+			// Test inserting the dock widget tab at a given position instead
+			// of appending it. This function inserts the new dock widget as
+			// first tab
+			d->DockManager->addDockWidgetTabToArea(DockWidget, EditorArea, 0);
+		}
+		else
+		{
+			d->DockManager->setConfigFlag(ads::CDockManager::EqualSplitOnInsertion, true);
+			d->DockManager->addDockWidget(ads::RightDockWidgetArea, DockWidget, EditorArea);
+		}
+	}
+	else
+	{
+		if (d->LastCreatedFloatingEditor)
+		{
+			d->DockManager->addDockWidget(ads::RightDockWidgetArea, DockWidget, d->LastCreatedFloatingEditor->dockAreaWidget());
+		}
+		else
+		{
+			d->DockManager->addDockWidget(ads::TopDockWidgetArea, DockWidget);
+		}
+	}
+	d->LastDockedEditor = DockWidget;
 }
 
 
@@ -822,6 +953,20 @@ void CMainWindow::onEditorCloseRequested()
 }
 
 
+//============================================================================
+void CMainWindow::onImageViewerCloseRequested()
+{
+	auto DockWidget = qobject_cast<ads::CDockWidget*>(sender());
+	int Result = QMessageBox::question(this, "Close Image Viewer", QString("%1 "
+		"contains unsaved changes? Would you like to close it?")
+		.arg(DockWidget->windowTitle()));
+	if (QMessageBox::Yes == Result)
+	{
+		DockWidget->closeDockWidget();
+	}
+}
+
+
 //============================================================================
 void CMainWindow::createTable()
 {
@@ -856,3 +1001,59 @@ void CMainWindow::toggleDockWidgetWindowTitle()
 	d->WindowTitleTestDockWidget->setWindowTitle(Title);
 }
 
+
+//============================================================================
+void CMainWindow::applyVsStyle()
+{
+	QFile StyleSheetFile(":adsdemo/res/visual_studio_light.css");
+	StyleSheetFile.open(QIODevice::ReadOnly);
+	QTextStream StyleSheetStream(&StyleSheetFile);
+	auto Stylesheet = StyleSheetStream.readAll();
+	StyleSheetFile.close();
+	d->DockManager->setStyleSheet(Stylesheet);
+}
+
+
+//============================================================================
+void CMainWindow::createImageViewer()
+{
+	QAction* a = qobject_cast<QAction*>(sender());
+	qDebug() << "createImageViewer " << a->text();
+
+	auto DockWidget = d->createImageViewer();
+	DockWidget->setFeature(ads::CDockWidget::DockWidgetDeleteOnClose, true);
+	DockWidget->setFeature(ads::CDockWidget::DockWidgetForceCloseWithArea, true);
+	DockWidget->setFeature(ads::CDockWidget::CustomCloseHandling, true);
+	DockWidget->resize(QSize(640, 480));
+	connect(DockWidget, &ads::CDockWidget::closeRequested, this,
+		&CMainWindow::onImageViewerCloseRequested);
+
+	if (a->text().startsWith("Floating"))
+	{
+		auto FloatingWidget = d->DockManager->addDockWidgetFloating(DockWidget);
+		FloatingWidget->move(QPoint(20, 20));
+	}
+	else if (a->text().startsWith("Docked"))
+	{
+		d->DockManager->addDockWidget(ads::RightDockWidgetArea, DockWidget);
+	}
+	else if (a->text().startsWith("Pinned"))
+	{
+		d->DockManager->addAutoHideDockWidget(ads::SideBarLeft, DockWidget);
+	}
+}
+
+
+//============================================================================
+void CMainWindow::lockWorkspace(bool Value)
+{
+	if (Value)
+	{
+		d->DockManager->lockDockWidgetFeaturesGlobally();
+	}
+	else
+	{
+		d->DockManager->lockDockWidgetFeaturesGlobally(ads::CDockWidget::NoDockWidgetFeatures);
+	}
+}
+
