@@ -38,11 +38,18 @@
 
 #include "DockAreaWidget.h"
 #include "DockAreaTitleBar.h"
+#include "DockContainerWidget.h"
+#include "AutoHideSideBar.h"
+#include "DockManager.h"
+#include "DockAreaTabBar.h"
 
 #include <iostream>
 
 namespace ads
 {
+static const int AutoHideAreaWidth = 32;
+static const int AutoHideAreaMouseZone = 8;
+static const int InvalidTabIndex = -2;
 
 /**
  * Private data class of CDockOverlay
@@ -57,11 +64,23 @@ struct DockOverlayPrivate
 	bool DropPreviewEnabled = true;
 	CDockOverlay::eMode Mode = CDockOverlay::ModeDockAreaOverlay;
 	QRect DropAreaRect;
+	int TabIndex = InvalidTabIndex;
 
 	/**
 	 * Private data constructor
 	 */
 	DockOverlayPrivate(CDockOverlay* _public) : _this(_public) {}
+
+	/**
+	 * Returns the overlay width / height depending on the visibility
+	 * of the sidebar
+	 */
+	int sideBarOverlaySize(SideBarLocation sideBarLocation);
+
+	/**
+	 * The area where the mouse is considered in the sidebar
+	 */
+	int sideBarMouseZone(SideBarLocation sideBarLocation);
 };
 
 /**
@@ -139,7 +158,7 @@ struct DockOverlayCrossPrivate
      */
     qreal dropIndicatiorWidth(QLabel* l) const
     {
-    #ifdef Q_OS_LINUX
+    #if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
         Q_UNUSED(l)
         return 40;
     #else
@@ -155,8 +174,20 @@ struct DockOverlayCrossPrivate
 		QLabel* l = new QLabel();
 		l->setObjectName("DockWidgetAreaLabel");
 
-        const qreal metric = dropIndicatiorWidth(l);
-		const QSizeF size(metric, metric);
+        qreal metric = dropIndicatiorWidth(l);
+		QSizeF size(metric, metric);
+		if (internal::isSideBarArea(DockWidgetArea))
+        {
+			auto SideBarLocation = internal::toSideBarLocation(DockWidgetArea);
+			if (internal::isHorizontalSideBarLocation(SideBarLocation))
+			{
+				size.setHeight(size.height() / 2);
+			}
+			else
+			{
+				size.setWidth(size.width() / 2);
+			}
+        }
 
 		l->setPixmap(createHighDpiDropIndicatorPixmap(size, DockWidgetArea, Mode));
 		l->setWindowFlags(Qt::Tool | Qt::FramelessWindowHint);
@@ -182,6 +213,11 @@ struct DockOverlayCrossPrivate
 	{
 		QColor borderColor = iconColor(CDockOverlayCross::FrameColor);
 		QColor backgroundColor = iconColor(CDockOverlayCross::WindowBackgroundColor);
+		QColor overlayColor = iconColor(CDockOverlayCross::OverlayColor);
+		if (overlayColor.alpha() == 255)
+		{
+			overlayColor.setAlpha(64);
+		}
 
 #if QT_VERSION >= 0x050600
 		double DevicePixelRatio = _this->window()->devicePixelRatioF();
@@ -239,22 +275,22 @@ struct DockOverlayCrossPrivate
 		}
 
 		QSizeF baseSize = baseRect.size();
-		if (CDockOverlay::ModeContainerOverlay == Mode && DockWidgetArea != CenterDockWidgetArea)
+		bool IsOuterContainerArea = (CDockOverlay::ModeContainerOverlay == Mode)
+			&& (DockWidgetArea != CenterDockWidgetArea)
+			&& !internal::isSideBarArea(DockWidgetArea);
+
+		if (IsOuterContainerArea)
 		{
 			baseRect = areaRect;
 		}
 
 		p.fillRect(baseRect, backgroundColor);
+
 		if (areaRect.isValid())
 		{
 			pen = p.pen();
 			pen.setColor(borderColor);
-			QColor Color = iconColor(CDockOverlayCross::OverlayColor);
-			if (Color.alpha() == 255)
-			{
-				Color.setAlpha(64);
-			}
-			p.setBrush(Color);
+			p.setBrush(overlayColor);
 			p.setPen(Qt::NoPen);
 			p.drawRect(areaRect);
 
@@ -267,6 +303,7 @@ struct DockOverlayCrossPrivate
 		}
 		p.restore();
 
+
 		p.save();
 		// Draw outer border
 		pen = p.pen();
@@ -282,8 +319,9 @@ struct DockOverlayCrossPrivate
 		p.drawRect(FrameRect);
 		p.restore();
 
+
 		// Draw arrow for outer container drop indicators
-		if (CDockOverlay::ModeContainerOverlay == Mode && DockWidgetArea != CenterDockWidgetArea)
+		if (IsOuterContainerArea)
 		{
 			QRectF ArrowRect;
 			ArrowRect.setSize(baseSize);
@@ -326,6 +364,38 @@ struct DockOverlayCrossPrivate
 };
 
 
+//============================================================================
+int DockOverlayPrivate::sideBarOverlaySize(SideBarLocation sideBarLocation)
+{
+	auto Container = qobject_cast<CDockContainerWidget*>(TargetWidget.data());
+	auto SideBar = Container->autoHideSideBar(sideBarLocation);
+	if (!SideBar || !SideBar->isVisibleTo(Container))
+	{
+		return AutoHideAreaWidth;
+	}
+	else
+	{
+		return (SideBar->orientation() == Qt::Horizontal) ? SideBar->height() : SideBar->width();
+	}
+}
+
+
+//============================================================================
+int DockOverlayPrivate::sideBarMouseZone(SideBarLocation sideBarLocation)
+{
+	auto Container = qobject_cast<CDockContainerWidget*>(TargetWidget.data());
+	auto SideBar = Container->autoHideSideBar(sideBarLocation);
+	if (!SideBar || !SideBar->isVisibleTo(Container))
+	{
+		return AutoHideAreaMouseZone;
+	}
+	else
+	{
+		return (SideBar->orientation() == Qt::Horizontal) ? SideBar->height() : SideBar->width();
+	}
+}
+
+
 //============================================================================
 CDockOverlay::CDockOverlay(QWidget* parent, eMode Mode) :
 	QFrame(parent),
@@ -333,7 +403,7 @@ CDockOverlay::CDockOverlay(QWidget* parent, eMode Mode) :
 {
 	d->Mode = Mode;
 	d->Cross = new CDockOverlayCross(this);
-#ifdef Q_OS_LINUX
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
 	setWindowFlags(Qt::Tool | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint);
 #else
 	setWindowFlags(Qt::Tool | Qt::FramelessWindowHint);
@@ -359,12 +429,26 @@ CDockOverlay::~CDockOverlay()
 void CDockOverlay::setAllowedAreas(DockWidgetAreas areas)
 {
 	if (areas == d->AllowedAreas)
+	{
 		return;
+	}
 	d->AllowedAreas = areas;
 	d->Cross->reset();
 }
 
 
+//============================================================================
+void CDockOverlay::setAllowedArea(DockWidgetArea area, bool Enable)
+{
+	auto AreasOld = d->AllowedAreas;
+	d->AllowedAreas.setFlag(area, Enable);
+	if (AreasOld != d->AllowedAreas)
+	{
+		d->Cross->reset();
+	}
+}
+
+
 //============================================================================
 DockWidgetAreas CDockOverlay::allowedAreas() const
 {
@@ -375,22 +459,68 @@ DockWidgetAreas CDockOverlay::allowedAreas() const
 //============================================================================
 DockWidgetArea CDockOverlay::dropAreaUnderCursor() const
 {
+	d->TabIndex = InvalidTabIndex;
+	if (!d->TargetWidget)
+	{
+		return InvalidDockWidgetArea;
+	}
+
 	DockWidgetArea Result = d->Cross->cursorLocation();
 	if (Result != InvalidDockWidgetArea)
 	{
 		return Result;
 	}
 
-	CDockAreaWidget* DockArea = qobject_cast<CDockAreaWidget*>(d->TargetWidget.data());
-	if (!DockArea)
+	auto CursorPos = QCursor::pos();
+	auto DockArea = qobject_cast<CDockAreaWidget*>(d->TargetWidget.data());
+	if (!DockArea && CDockManager::autoHideConfigFlags().testFlag(CDockManager::AutoHideFeatureEnabled))
+	{
+		auto Rect = rect();
+		const QPoint pos = mapFromGlobal(QCursor::pos());
+		if ((pos.x() < d->sideBarMouseZone(SideBarLeft))
+		  && d->AllowedAreas.testFlag(LeftAutoHideArea))
+		{
+			Result = LeftAutoHideArea;
+		}
+		else if (pos.x() > (Rect.width() - d->sideBarMouseZone(SideBarRight))
+			  && d->AllowedAreas.testFlag(RightAutoHideArea))
+		{
+			Result = RightAutoHideArea;
+		}
+		else if (pos.y() < d->sideBarMouseZone(SideBarTop)
+			&& d->AllowedAreas.testFlag(TopAutoHideArea))
+		{
+			Result = TopAutoHideArea;
+		}
+		else if (pos.y() > (Rect.height() - d->sideBarMouseZone(SideBarBottom))
+			&& d->AllowedAreas.testFlag(BottomAutoHideArea))
+		{
+			Result = BottomAutoHideArea;
+		}
+
+		auto SideBarLocation = ads::internal::toSideBarLocation(Result);
+		if (SideBarLocation != SideBarNone)
+		{
+			auto Container = qobject_cast<CDockContainerWidget*>(d->TargetWidget.data());
+			auto SideBar = Container->autoHideSideBar(SideBarLocation);
+			if (SideBar->isVisible())
+			{
+				d->TabIndex = SideBar->tabInsertIndexAt(SideBar->mapFromGlobal(CursorPos));
+			}
+		}
+		return Result;
+	}
+	else if (!DockArea)
 	{
 		return Result;
 	}
 
 	if (DockArea->allowedAreas().testFlag(CenterDockWidgetArea)
 	 && !DockArea->titleBar()->isHidden()
-	 && DockArea->titleBarGeometry().contains(DockArea->mapFromGlobal(QCursor::pos())))
+	 && DockArea->titleBarGeometry().contains(DockArea->mapFromGlobal(CursorPos)))
 	{
+		auto TabBar = DockArea->titleBar()->tabBar();
+		d->TabIndex = TabBar->tabInsertIndexAt(TabBar->mapFromGlobal(CursorPos));
 		return CenterDockWidgetArea;
 	}
 
@@ -398,6 +528,13 @@ DockWidgetArea CDockOverlay::dropAreaUnderCursor() const
 }
 
 
+//============================================================================
+int CDockOverlay::tabIndexUnderCursor() const
+{
+	return d->TabIndex;
+}
+
+
 //============================================================================
 DockWidgetArea CDockOverlay::visibleDropAreaUnderCursor() const
 {
@@ -431,6 +568,7 @@ DockWidgetArea CDockOverlay::showOverlay(QWidget* target)
 	d->LastLocation = InvalidDockWidgetArea;
 
 	// Move it over the target.
+	hide();
 	resize(target->size());
 	QPoint TopLeft = target->mapToGlobal(target->rect().topLeft());
 	move(TopLeft);
@@ -470,6 +608,7 @@ bool CDockOverlay::dropPreviewEnabled() const
 void CDockOverlay::paintEvent(QPaintEvent* event)
 {
 	Q_UNUSED(event);
+
 	// Draw rect based on location
 	if (!d->DropPreviewEnabled)
 	{
@@ -489,8 +628,13 @@ void CDockOverlay::paintEvent(QPaintEvent* event)
 	case BottomDockWidgetArea: r.setY(r.height() * (1 - 1 / Factor)); break;
 	case LeftDockWidgetArea: r.setWidth(r.width() / Factor); break;
 	case CenterDockWidgetArea: r = rect();break;
+	case LeftAutoHideArea: r.setWidth(d->sideBarOverlaySize(SideBarLeft)); break;
+	case RightAutoHideArea: r.setX(r.width() - d->sideBarOverlaySize(SideBarRight)); break;
+	case TopAutoHideArea: r.setHeight(d->sideBarOverlaySize(SideBarTop)); break;
+	case BottomAutoHideArea: r.setY(r.height() - d->sideBarOverlaySize(SideBarBottom)); break;
 	default: return;
 	}
+
 	QPainter painter(this);
     QColor Color = palette().color(QPalette::Active, QPalette::Highlight);
     QPen Pen = painter.pen();
@@ -594,7 +738,7 @@ CDockOverlayCross::CDockOverlayCross(CDockOverlay* overlay) :
 	d(new DockOverlayCrossPrivate(this))
 {
 	d->DockOverlay = overlay;
-#ifdef Q_OS_LINUX
+#if defined(Q_OS_UNIX) && !defined(Q_OS_MACOS)
 	setWindowFlags(Qt::Tool | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | Qt::X11BypassWindowManagerHint);
 #else
 	setWindowFlags(Qt::Tool | Qt::FramelessWindowHint);
@@ -626,6 +770,7 @@ void CDockOverlayCross::setupOverlayCross(CDockOverlay::eMode Mode)
 	areaWidgets.insert(BottomDockWidgetArea, d->createDropIndicatorWidget(BottomDockWidgetArea, Mode));
 	areaWidgets.insert(LeftDockWidgetArea, d->createDropIndicatorWidget(LeftDockWidgetArea, Mode));
 	areaWidgets.insert(CenterDockWidgetArea, d->createDropIndicatorWidget(CenterDockWidgetArea, Mode));
+
 #if QT_VERSION >= 0x050600
 	d->LastDevicePixelRatio = devicePixelRatioF();
 #else
